@@ -1,6 +1,11 @@
 """
-:mod:`fastf1.api` - Api module
-==============================
+Api Functions - :mod:`fastf1.api`
+=================================
+
+.. note:: The functions listed here are primarily for internal use within
+    FastF1. While you can use these functions directly, it is usually
+    better to use the functionality provided by the data objects
+    in :mod:`fastf1.core` instead.
 
 A collection of functions to interface with the F1 web api.
 
@@ -19,25 +24,24 @@ A collection of functions to interface with the F1 web api.
    parse
 
 """
-import datetime
-import json
 import base64
-import zlib
+import datetime
+import functools
+import json
+import logging
 import os
 import pickle
+import zlib
+
+import numpy as np
+import pandas as pd
 import requests
 import requests_cache
-import logging
-import pandas as pd
-import numpy as np
 
 from fastf1.utils import recursive_dict_get, to_timedelta, to_datetime
 
-base_url = 'https://livetiming.formula1.com'
 
-requests_session = None
-"""Can be used to create a custom requests session which is then used by
-all api calls"""
+base_url = 'https://livetiming.formula1.com'
 
 headers = {
   'Host': 'livetiming.formula1.com',
@@ -76,24 +80,42 @@ class Cache:
     """Pickle and requests based API cache.
 
     The parsed API data will be saved as a pickled object.
-    Raw GET and POST requests are cached by requests-cache in a sqlite db.
+    Raw GET requests are cached in a sqlite db using the 'requests-cache'
+    module.
+
+    Caching should almost always be enabled to speed up the runtime of your
+    scripts and to prevent exceeding the rate limit of api servers.
+    FastF1 will print an annoyingly obnoxious warning message if you do not
+    enable caching.
 
     The cache has two "stages".
 
-        - Stage 1: Caching of raw GET and POST requests. This works for all requests. The returned data is unlikely
-          to ever change.
-        - Stage 2: Caching of the parsed data. This saves a lot of time when running as parsing of the data is
-          computationally expensive. This data can change whenever the API parser code is updated.
-          Cache data is saved together with a version number. Updates of the code are automatically detected
-          (comparing version numbers). The cache is updated in case the version numbers of the cached
-          data and the code don't match. Stage 2 is only used for some api functions.
+        - Stage 1: Caching of raw GET requests. This works for all requests.
+          Cache control is employed to refresh the cached data periodically.
+        - Stage 2: Caching of the parsed data. This saves a lot of time when
+          running your scripts,  as parsing of the data is computationally
+          expensive. Stage 2 caching is only used for some api functions.
+
+    Most commonly, you will enable caching right at the beginning of your script:
+
+        >>> import fastf1
+        >>> fastf1.Cache.enable_cache('path/to/cache')  # doctest: +SKIP
+        # change cache directory to an exisitng empty directory on your machine
+        >>> session = fastf1.get_session(2021, 5, 'Q')
+        >>> # ...
+
+    Note that you should always enable caching except for very rare
+    circumstances which are usually limited to doing core developement
+    on FastF1.
     """
     _CACHE_DIR = ''
-    _API_CORE_VERSION = 1  # version of the api parser code (unrelated to release version number)
+    _API_CORE_VERSION = 3  # version of the api parser code (unrelated to release version number)
     _IGNORE_VERSION = False
     _FORCE_RENEW = False
 
+    _requests_session = None
     _has_been_warned = False  # flag to ensure that warning about disabled cache is logged once only
+    _tmp_disabled = False
 
     @classmethod
     def enable_cache(cls, cache_dir, ignore_version=False, force_renew=False, use_requests_cache=True):
@@ -112,17 +134,44 @@ class Cache:
         cls._IGNORE_VERSION = ignore_version
         cls._FORCE_RENEW = force_renew
         if use_requests_cache:
-            cls._install_requests_cache(cache_dir, force_renew=force_renew)
+            cls._requests_session = requests_cache.CachedSession(
+                cache_name=os.path.join(cache_dir, 'fastf1_http_cache'),
+                backend='sqlite',
+                allowable_methods=('GET', 'POST'),
+                expire_after=datetime.timedelta(hours=12),
+                cache_control=True,
+                stale_if_error=True
+            )
+            if force_renew:
+                cls._requests_session.cache.clear()
 
-    @staticmethod
-    def _install_requests_cache(cache_dir, *, force_renew=False):
-        requests_cache.install_cache(
-            os.path.join(cache_dir, 'fastf1_http_cache'),
-            allowable_methods=('GET', 'POST'),
-            expire_after=datetime.timedelta(hours=12)
-        )
-        if force_renew:
-            requests_cache.clear()
+    @classmethod
+    def requests_get(cls, *args, **kwargs):
+        """Wraps `requests.Session().get()` with caching if enabled.
+
+        All GET requests that require caching should be performed through this
+        wrapper. Caching will be done if the module-wide cache has been
+        enabled. Else, `requests.Session().get()` will be called without any
+        caching.
+        """
+        cls._show_not_enabled_warning()
+        if (cls._requests_session is None) or cls._tmp_disabled:
+            return requests.get(*args, **kwargs)
+        return cls._requests_session.get(*args, **kwargs)
+
+    @classmethod
+    def requests_post(cls, *args, **kwargs):
+        """Wraps `requests.Session().post()` with caching if enabled.
+
+        All POST requests that require caching should be performed through this
+        wrapper. Caching will be done if the module-wide cache has been
+        enabled. Else, `requests.Session().get()` will be called without any
+        caching.
+        """
+        cls._show_not_enabled_warning()
+        if (cls._requests_session is None) or cls._tmp_disabled:
+            return requests.post(*args, **kwargs)
+        return cls._requests_session.post(*args, **kwargs)
 
     @classmethod
     def clear_cache(cls, cache_dir, deep=False):
@@ -166,16 +215,13 @@ class Cache:
         Returns:
             The wrapped function
         """
-        def cached_api_request(api_path, response=None, livedata=None):
-            if cls._CACHE_DIR:
-                # caching is enabled, extend the cache dir path using the api path
-                cache_dir_path = os.path.join(cls._CACHE_DIR, api_path[8:])  # remove leading '/static/' from api path
-                if not os.path.exists(cache_dir_path):
-                    os.makedirs(cache_dir_path)  # create subfolders if they don't yet exist
+        @functools.wraps(func)
+        def _cached_api_request(api_path, response=None, livedata=None):
+            if cls._CACHE_DIR and not cls._tmp_disabled:
+                # caching is enabled
+                func_name = str(func.__name__)
+                cache_file_path = cls._get_cache_file_path(api_path, func_name)
 
-                # create cache file path for this specific request by extending cache dir path with file named after
-                # requesting api function
-                cache_file_path = cache_dir_path + str(func.__name__) + '.ff1pkl'
                 if os.path.isfile(cache_file_path):
                     # file exists already, try to load it
                     try:
@@ -186,52 +232,155 @@ class Cache:
                         # after it was updated
                         cached = None
 
-                    if (cached is not None and
-                            ((cached['version'] == cls._API_CORE_VERSION) or
-                             cls._IGNORE_VERSION)
-                            and not cls._FORCE_RENEW):
-                        # was created with same version or version is ignored
-                        logging.info(f"Using cached data for {str(func.__name__)}")
+                    if cached is not None and cls._data_ok_for_use(cached):
+                        # cached data is ok for use, return it
+                        logging.info(f"Using cached data for {func_name}")
                         return cached['data']
 
-                    else:  # created with different version or force renew --> download again and update
-                        logging.info(f"Updating cache for {str(func.__name__)}...")
-                        data = func(api_path, response=response, livedata=livedata)
-                        if data is not None:
-                            new_cached = {'version': cls._API_CORE_VERSION, 'data': data}
-                            pickle.dump(new_cached, open(cache_file_path, 'wb'))
-                            logging.info("Cache updated!")
-                        else:  # downloading data failed, cannot proceed because of missing critical data
-                            logging.critical("Failed to update cache! No data received! Cannot continue!")
-                            exit()
-                        return data
-
-                else:  # cached data does not yet exist for this request, try to download and create cache
-                    logging.info(f"No cached data found for {str(func.__name__)}. Loading data...")
-                    data = func(api_path, response=response, livedata=livedata)
-                    if data is not None:
-                        new_cached = {'version': cls._API_CORE_VERSION, 'data': data}
-                        pickle.dump(new_cached, open(cache_file_path, 'wb'))
-                        logging.info("Data has been written to cache!")
-                        return data
                     else:
-                        logging.critical("Failed to load data!")
+                        # cached data needs to be downloaded again and updated
+                        logging.info(f"Updating cache for {func_name}...")
+                        data = func(
+                            api_path, response=response, livedata=livedata
+                        )
+
+                        if data is not None:
+                            cls._write_cache(data, cache_file_path)
+                            logging.info("Cache updated!")
+                            return data
+
+                        logging.critical(
+                            "A cache update is required but the data failed "
+                            "to download. Cannot continue!\nYou may force to "
+                            "ignore a cache version mismatch by using the "
+                            "`ignore_version=True` keyword when enabling the "
+                            "cache (not recommended)."
+                        )
                         exit()
 
+                else:  # cached data does not yet exist for this api request
+                    logging.info(f"No cached data found for {func_name}. "
+                                 f"Loading data...")
+                    data = func(
+                        api_path, response=response, livedata=livedata
+                    )
+                    if data is not None:
+                        cls._write_cache(data, cache_file_path)
+                        logging.info("Data has been written to cache!")
+                        return data
+
+                    logging.critical("Failed to load data!")
+                    exit()
+
             else:  # cache was not enabled
-                if not cls._has_been_warned:  # warn only once
-                    logging.warning("\n\nNO CACHE! Api caching has not been enabled! \n\t"
-                                    "It is highly recommended to enable this feature for much faster data loading!\n\t"
-                                    "Use `fastf1.Cache.enable_cache('path/to/cache/')`\n")
-
-                    cls._has_been_warned = True
-
+                if not cls._tmp_disabled:
+                    cls._show_not_enabled_warning()
                 return func(api_path, response=response, livedata=livedata)
 
-        wrapped = cached_api_request
-        wrapped.__doc__ = func.__doc__  # necessary to make docstrings work
+        return _cached_api_request
 
-        return wrapped
+    @classmethod
+    def _get_cache_file_path(cls, api_path, name):
+        # extend the cache dir path using the api path and a file name
+        # leading '/static/' is dropped form api path
+        cache_dir_path = os.path.join(cls._CACHE_DIR, api_path[8:])
+        if not os.path.exists(cache_dir_path):
+            # create subfolders if they don't yet exist
+            os.makedirs(cache_dir_path)
+
+        file_name = name + '.ff1pkl'
+        cache_file_path = os.path.join(cache_dir_path, file_name)
+        return cache_file_path
+
+    @classmethod
+    def _data_ok_for_use(cls, cached):
+        # check if cached data is ok or needs to be downloaded again
+        if cls._FORCE_RENEW:
+            return False
+        elif cls._IGNORE_VERSION:
+            return True
+        elif cached['version'] == cls._API_CORE_VERSION:
+            return True
+        return False
+
+    @classmethod
+    def _write_cache(cls, data, cache_file_path, **kwargs):
+        new_cached = dict(
+            **{'version': cls._API_CORE_VERSION, 'data': data},
+            **kwargs
+        )
+        with open(cache_file_path, 'wb') as cache_file_obj:
+            pickle.dump(new_cached, cache_file_obj)
+
+    @classmethod
+    def _show_not_enabled_warning(cls):
+        if not cls._CACHE_DIR and not cls._has_been_warned:
+            # warn only once and only if cache is not enabled
+            logging.warning(
+                "\n\nNO CACHE! Api caching has not been enabled! \n\t"
+                "It is highly recommended to enable this feature for much "
+                "faster data loading!\n\t"
+                "Use `fastf1.Cache.enable_cache('path/to/cache/')`\n")
+
+            cls._has_been_warned = True
+
+    @classmethod
+    def disabled(cls):
+        """Returns a context manager object that creates a context within
+        which the cache is temporarily disabled.
+
+        Example::
+
+            with Cache.disabled():
+                # no caching takes place here
+                ...
+
+        .. note::
+            The context manager is not multithreading-safe
+        """
+        return _NoCacheContext()
+
+    @classmethod
+    def set_disabled(cls):
+        """Disable the cache while keeping the configuration intact.
+
+        This disables stage 1 and stage 2 caching!
+
+        You can enable the cache at any time using :func:`set_enabled`
+
+        .. note:: You may prefer to use :func:`disabled` to get a context
+            manager object and disable the cache only within a specific
+            context.
+
+        .. note::
+            This function is not multithreading-safe
+        """
+        cls._tmp_disabled = True
+
+    @classmethod
+    def set_enabled(cls):
+        """Enable the cache after it has been disabled with
+        :func:`set_disabled`.
+
+        .. warning::
+            To enable the cache it needs to be configured properly. You need
+            to call :func`enable_cache` once to enable the cache initially.
+            :func:`set_enabled` and :func:`set_disabled` only serve to
+            (temporarily) disable the cache for specific parts of code that
+            should be run without caching.
+
+        .. note::
+            This function is not multithreading-safe
+        """
+        cls._tmp_disabled = False
+
+
+class _NoCacheContext:
+    def __enter__(self):
+        Cache.set_disabled()
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        Cache.set_enabled()
 
 
 def make_path(wname, wdate, sname, sdate):
@@ -288,9 +437,9 @@ def timing_data(path, response=None, livedata=None):
 
 
     Args:
-        path: api path base string (see :func:`make_path`)
+        path: api path base string (usually ``Session.api_path``)
         response (optional): api response can be passed if data was already downloaded
-        livedata: An instance of :class:`fastf1.livetiming.data.LivetimingData`
+        livedata: An instance of :class:`fastf1.livetiming.data.LiveTimingData`
             to use as a source instead of the api
 
     Returns:
@@ -337,8 +486,10 @@ def timing_data(path, response=None, livedata=None):
         logging.info("Fetching timing data...")
         response = fetch_page(path, 'timing_data')
         if response is None:  # no response received
-            raise SessionNotAvailableError("No data for this session! Are you sure this session wasn't cancelled?")
-
+            raise SessionNotAvailableError(
+                "No data for this session! If this session only finished "
+                "recently, please try again in a few minutes."
+            )
     logging.info("Parsing timing data...")
 
     # split up response per driver for easier iteration and processing later
@@ -369,9 +520,6 @@ def timing_data(path, response=None, livedata=None):
     laps_data = pd.DataFrame(laps_data)
     stream_data = pd.DataFrame(stream_data)
 
-    if ((laps_data.to_numpy() == '') | (pd.isna(laps_data.to_numpy()))).all():  # if all values of the frame are nan/...
-        raise SessionNotAvailableError("No data for this session! Are you sure this session wasn't cancelled?")
-
     return laps_data, stream_data
 
 
@@ -689,9 +837,9 @@ def timing_app_data(path, response=None, livedata=None):
     frequently (for example 'Compound' only after tire changes).
 
     Args:
-        path (str): api path base string (see :func:`make_path`)
+        path (str): api path base string (usually ``Session.api_path``)
         response: Response as returned by :func:`fetch_page` can be passed if it was downloaded already.
-        livedata: An instance of :class:`fastf1.livetiming.data.LivetimingData` to use as a source instead of the api
+        livedata: An instance of :class:`fastf1.livetiming.data.LiveTimingData` to use as a source instead of the api
 
     Returns:
         A DataFrame contianing one column for each data channel as listed above.
@@ -705,7 +853,10 @@ def timing_app_data(path, response=None, livedata=None):
         logging.info("Fetching timing app data...")
         response = fetch_page(path, 'timing_app_data')
         if response is None:  # no response received
-            raise SessionNotAvailableError("No data for this session! Are you sure this session wasn't cancelled?")
+            raise SessionNotAvailableError(
+                "No data for this session! If this session only finished "
+                "recently, please try again in a few minutes."
+            )
 
     data = {'LapNumber': [], 'Driver': [], 'LapTime': [], 'Stint': [], 'TotalLaps': [], 'Compound': [], 'New': [],
             'TyresNotChanged': [], 'Time': [], 'LapFlags': [], 'LapCountTime': [], 'StartLaps': [], 'Outlap': []}
@@ -754,7 +905,7 @@ def car_data(path, response=None, livedata=None):
         - RPM (int)
         - Gear (int): [called 'nGear' in the data!]
         - Throttle (int): 0-100%
-        - Brake (int): 0-100% (don't trust brake too much)
+        - Brake (bool)
         - DRS (int): 0-14 (Odd DRS is Disabled, Even DRS is Enabled) (More Research Needed)
             | 0 =  Off
             | 1 =  Off
@@ -770,9 +921,9 @@ def car_data(path, response=None, livedata=None):
     car data do not line up. Resampling/interpolation is required to merge them.
 
     Args:
-        path (str): api path base string (see :func:`make_path`)
+        path (str): api path base string (usually ``Session.api_path``)
         response: Response as returned by :func:`fetch_page` can be passed if it was downloaded already.
-        livedata: An instance of :class:`fastf1.livetiming.data.LivetimingData` to use as a source instead of the api
+        livedata: An instance of :class:`fastf1.livetiming.data.LiveTimingData` to use as a source instead of the api
 
     Returns:
         | A dictionary containing one pandas DataFrame per driver. Dictionary keys are the driver's numbers as
@@ -792,11 +943,16 @@ def car_data(path, response=None, livedata=None):
         logging.info("Fetching car data...")
         response = fetch_page(path, 'car_data')
         if response is None:  # no response received
-            raise SessionNotAvailableError("No data for this session! Are you sure this session wasn't cancelled?")
+            raise SessionNotAvailableError(
+                "No data for this session! If this session only finished "
+                "recently, please try again in a few minutes."
+            )
 
     logging.info("Parsing car data...")
 
     channels = {'0': 'RPM', '2': 'Speed', '3': 'nGear', '4': 'Throttle', '5': 'Brake', '45': 'DRS'}
+    num_channels = ['RPM', 'Speed', 'nGear', 'Throttle', 'DRS']
+    bool_channels = ['Brake']
     columns = ['Time', 'Date', 'RPM', 'Speed', 'nGear', 'Throttle', 'Brake', 'DRS', 'Source']
     ts_length = 12  # length of timestamp: len('00:00:00:000')
 
@@ -842,14 +998,16 @@ def car_data(path, response=None, livedata=None):
     # create one dataframe per driver and check for the longest dataframe
     most_complete_ref = None
     for driver in data:
-        data[driver]['Source'] = ['car', ] * len(data[driver]['Date'])  # add source reference for each sample
+        # add source reference for each sample
+        data[driver]['Source'] = ['car', ] * len(data[driver]['Date'])
         data[driver] = pd.DataFrame(data[driver])  # convert dict to dataframe
         # check length of dataframe; sometimes there can be missing data
         if most_complete_ref is None or len(data[driver]['Date']) > len(most_complete_ref):
             most_complete_ref = data[driver]['Date']
 
-    # if everything is well, all dataframes should have the same length and no postprocessing is necessary
     for driver in data:
+        # if everything is well, all dataframes should have the same length
+        # and no postprocessing is necessary
         if len(data[driver]['Date']) < len(most_complete_ref):
             # there is missing data for this driver
             # extend the Date column and fill up missing telemetry values with
@@ -857,12 +1015,31 @@ def car_data(path, response=None, livedata=None):
             # correctly based on Session.t0_date anyways when creating Telemetry
             # instances in Session.load_telemetry
             index_df = pd.DataFrame(data={'Date': most_complete_ref})
-            data[driver] = data[driver].merge(index_df, how='outer').sort_values(by='Date').reset_index(drop=True)
-            data[driver].loc[:, channels.values()] = \
-                data[driver].loc[:, channels.values()].fillna(value=0, inplace=False).astype('int64')
+            data[driver] = data[driver]\
+                .merge(index_df, how='outer')\
+                .sort_values(by='Date')\
+                .reset_index(drop=True)
 
             logging.warning(f"Driver {driver: >2}: Car data is incomplete!")
 
+        # ensure that brake data is 'boolean-compatible' in case that this is
+        # ever changed
+        _unique_brake_values = data[driver].loc[:, 'Brake'].unique()
+        if ((_unique_brake_values > 0) & (_unique_brake_values < 100)).any():
+            logging.warning(f"Driver {driver: >2}: Raw brake data contains "
+                            f"non-boolean values!")
+
+        # convert to correct datatypes
+        data[driver].loc[:, num_channels] = data[driver] \
+            .loc[:, num_channels]\
+            .fillna(value=0, inplace=False)\
+            .astype('int64')
+
+        data[driver].loc[:, bool_channels] = data[driver] \
+            .loc[:, bool_channels]\
+            .fillna(value=False, inplace=False)\
+            .astype('bool')
+
     return data
 
 
@@ -881,9 +1058,9 @@ def position_data(path, response=None, livedata=None):
     car data do not line up. Resampling/interpolation is required to merge them.
 
     Args:
-        path (str): api path base string (see :func:`api.make_path`)
-        response: Response as returned by :func:`api.fetch_page` can be passed if it was downloaded already.
-        livedata: An instance of :class:`fastf1.livetiming.data.LivetimingData` to use as a source instead of the api
+        path (str): api path base string (usually ``Session.api_path``)
+        response: Response as returned by :func:`fetch_page` can be passed if it was downloaded already.
+        livedata: An instance of :class:`fastf1.livetiming.data.LiveTimingData` to use as a source instead of the api
 
     Returns:
         | A dictionary containing one pandas DataFrame per driver. Dictionary keys are the driver's numbers as
@@ -903,7 +1080,10 @@ def position_data(path, response=None, livedata=None):
         logging.info("Fetching position data...")
         response = fetch_page(path, 'position')
         if response is None:  # no response received
-            raise SessionNotAvailableError("No data for this session! Are you sure this session wasn't cancelled?")
+            raise SessionNotAvailableError(
+                "No data for this session! If this session only finished "
+                "recently, please try again in a few minutes."
+            )
 
     logging.info("Parsing position data...")
 
@@ -1009,9 +1189,9 @@ def track_status_data(path, response=None, livedata=None):
           will mark the actual end)
 
     Args:
-        path (str): api path base string (see :func:`api.make_path`)
-        response: Response as returned by :func:`api.fetch_page` can be passed if it was downloaded already.
-        livedata: An instance of :class:`fastf1.livetiming.data.LivetimingData` to use as a source instead of the api
+        path (str): api path base string (usually ``Session.api_path``)
+        response: Response as returned by :func:`fetch_page` can be passed if it was downloaded already.
+        livedata: An instance of :class:`fastf1.livetiming.data.LiveTimingData` to use as a source instead of the api
 
     Returns:
         A dictionary containing one key for each data channel and a list of values per key.
@@ -1027,7 +1207,10 @@ def track_status_data(path, response=None, livedata=None):
         logging.info("Fetching track status data...")
         response = fetch_page(path, 'track_status')
         if response is None:  # no response received
-            raise SessionNotAvailableError("No data for this session! Are you sure this session wasn't cancelled?")
+            raise SessionNotAvailableError(
+                "No data for this session! If this session only finished "
+                "recently, please try again in a few minutes."
+            )
 
     data = {'Time': [], 'Status': [], 'Message': []}
 
@@ -1057,9 +1240,9 @@ def session_status_data(path, response=None, livedata=None):
     A new value is sent every time the session status changes.
 
     Args:
-        path (str): api path base string (see :func:`api.make_path`)
-        response: Response as returned by :func:`api.fetch_page` can be passed if it was downloaded already.
-        livedata: An instance of :class:`fastf1.livetiming.data.LivetimingData` to use as a source instead of the api
+        path (str): api path base string (usually ``Session.api_path``)
+        response: Response as returned by :func:`fetch_page` can be passed if it was downloaded already.
+        livedata: An instance of :class:`fastf1.livetiming.data.LiveTimingData` to use as a source instead of the api
 
     Returns:
         A dictionary containing one key for each data channel and a list of values per key.
@@ -1075,7 +1258,10 @@ def session_status_data(path, response=None, livedata=None):
         logging.info("Fetching session status data...")
         response = fetch_page(path, 'session_status')
         if response is None:  # no response received
-            raise SessionNotAvailableError("No data for this session! Are you sure this session wasn't cancelled?")
+            raise SessionNotAvailableError(
+                "No data for this session! If this session only finished "
+                "recently, please try again in a few minutes."
+            )
 
     data = {'Time': [], 'Status': []}
 
@@ -1103,10 +1289,10 @@ def driver_info(path, response=None, livedata=None):
         'HeadshotUrl']`
 
     Args:
-        path (str): api path base string (see :func:`api.make_path`)
-        response: Response as returned by :func:`api.fetch_page`
+        path (str): api path base string (usually ``Session.api_path``)
+        response: Response as returned by :func:`fetch_page`
             can be passed if it was downloaded already.
-        livedata: An instance of :class:`fastf1.livetiming.data.LivetimingData`
+        livedata: An instance of :class:`fastf1.livetiming.data.LiveTimingData`
             to use as a source instead of the api
 
     Returns:
@@ -1124,10 +1310,28 @@ def driver_info(path, response=None, livedata=None):
         logging.info("Fetching driver list...")
         response = fetch_page(path, 'driver_list')
         if response is None:  # no response received
-            raise SessionNotAvailableError("No data for this session! Are you sure this session wasn't cancelled?")
+            raise SessionNotAvailableError(
+                "No data for this session! If this session only finished "
+                "recently, please try again in a few minutes."
+            )
 
+    # search for the correct entries that contain driver and team info
+    drv_idx = None
+    team_idx = None
+    for i, entry in enumerate(response):
+        if 'RacingNumber' in str(entry):
+            drv_idx = i
+        if 'TeamName' in str(entry):
+            team_idx = i
+        if drv_idx and team_idx:
+            break
+
+    # parse data
     try:
-        drv_info = response[0][1]
+        drv_info = response[drv_idx][1]
+        team_info = response[team_idx][1]
+        for drv in drv_info:
+            drv_info[drv].update(team_info.get(drv, {}))
         if not len(drv_info) or not isinstance(drv_info, dict):
             return dict()
         if 'RacingNumber' not in list(drv_info.values())[0]:
@@ -1156,11 +1360,11 @@ def weather_data(path, response=None, livedata=None):
     Weather data is updated once per minute.
 
     Args:
-        path (str): api path base string (see :func:`api.make_path`)
-        response: Response as returned by :func:`api.fetch_page` can
+        path (str): api path base string (usually ``Session.api_path``)
+        response: Response as returned by :func:`fetch_page` can
             be passed if it was downloaded already.
         livedata: An instance of
-            :class:`fastf1.livetiming.data.LivetimingData`
+            :class:`fastf1.livetiming.data.LiveTimingData`
             to use as a source instead of the api
 
     Returns:
@@ -1179,7 +1383,10 @@ def weather_data(path, response=None, livedata=None):
         logging.info("Fetching weather data...")
         response = fetch_page(path, 'weather_data')
         if response is None:  # no response received
-            raise SessionNotAvailableError("No data for this session! Are you sure this session wasn't cancelled?")
+            raise SessionNotAvailableError(
+                "No data for this session! If this session only finished "
+                "recently, please try again in a few minutes."
+            )
 
     data = {
         'Time': [], 'AirTemp': [], 'Humidity': [], 'Pressure': [],
@@ -1188,7 +1395,9 @@ def weather_data(path, response=None, livedata=None):
 
     data_keys = ('AirTemp', 'Humidity', 'Pressure', 'Rainfall',
                  'TrackTemp', 'WindDirection', 'WindSpeed')
-    data_dtypes = (float, float, float, bool, float, int, float)
+    converters = (float, float, float,
+                  lambda v: True if v == '1' else False,  # rain: str -> bool
+                  float, int, float)
 
     for entry in response:
         if len(entry) < 2:
@@ -1198,12 +1407,12 @@ def weather_data(path, response=None, livedata=None):
             continue
 
         data['Time'].append(to_timedelta(entry[0]))
-        for key, dtype in zip(data_keys, data_dtypes):
+        for key, conv in zip(data_keys, converters):
             try:
-                data[key].append(dtype(row[key]))
+                data[key].append(conv(row[key]))
             except (KeyError, ValueError):
                 # type conversion failed or key is missing
-                data[key].append(dtype(0))
+                data[key].append(conv(0))
 
     return data
 
@@ -1213,7 +1422,7 @@ def fetch_page(path, name):
     to parse json or decode known messages is made.
 
     Args:
-        path (str): api path base string (see :func:`api.make_path`)
+        path (str): api path base string (usually ``Session.api_path``)
         name (str): page name (see :attr:`pages` for all known pages)
 
     Returns:
@@ -1226,11 +1435,7 @@ def fetch_page(path, name):
     page = pages[name]
     is_stream = 'jsonStream' in page
     is_z = '.z.' in page
-    if requests_session is None:
-        session = requests.session()
-    else:
-        session = requests_session
-    r = session.get(base_url + path + pages[name], headers=headers)
+    r = Cache.requests_get(base_url + path + pages[name], headers=headers)
     if r.status_code == 200:
         raw = r.content.decode('utf-8-sig')
         if is_stream:
